/*
Copyright (C) 2013 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */
package com.clustercontrol.cloud.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import com.clustercontrol.cloud.commons.util.MessageManager;
import com.clustercontrol.util.Messages;

public class CreateSecurityGroupDialog extends Dialog {
	private MessageManager bundle_dialog_name = MessageManager.getInstance("dialog_name");
	private MessageManager bundle_dialog_label = MessageManager.getInstance("dialog_label");
	private MessageManager bundle_messages = MessageManager.getInstance("messages");

	private Text txtSecurityGroupName;
	private Text txtDescription;
	private Text txtRegion;
	private String securityGroupName;
	private String description;
	
	private String regionName;

	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public CreateSecurityGroupDialog(Shell parentShell) {
		super(parentShell);
	}

	public void setRegionName(String regionName) {
		this.regionName = regionName;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);
		container.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		GridLayout gl_container = new GridLayout(2, false);
		gl_container.horizontalSpacing = 16;
		container.setLayout(gl_container);
		GridData gd_container = new GridData(GridData.FILL_BOTH);
		gd_container.widthHint = 614;
		container.setLayoutData(gd_container);
		
		Label lblRegion = new Label(container, SWT.NONE);
		lblRegion.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblRegion.setText(bundle_dialog_label.getString("dialog.create_security_group.region_title"));
		
		txtRegion = new Text(container, SWT.BORDER);
		txtRegion.setEnabled(false);
		txtRegion.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);
		
		Label lblSecurityGroupName = new Label(container, SWT.NONE);
		lblSecurityGroupName.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblSecurityGroupName.setText(bundle_dialog_label.getString("dialog.create_security_group.security_group_name_title"));
		
		txtSecurityGroupName = new Text(container, SWT.BORDER);
		GridData gd_SecurityGroupName = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_SecurityGroupName.widthHint = 203;
		txtSecurityGroupName.setLayoutData(gd_SecurityGroupName);
		
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);
		
		Label lblDescription = new Label(container, SWT.NONE);
		lblDescription.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblDescription.setText(bundle_dialog_label.getString("dialog.create_security_group.description_title"));
		
		txtDescription = new Text(container, SWT.BORDER);
		GridData gd_Description = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_Description.widthHint = 146;
		txtDescription.setLayoutData(gd_Description);
		
		if (regionName != null) {
			txtRegion.setText(regionName);
		}
		
		return area;
	}

	/**
	 * Create contents of the button bar.
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				false);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(640, 480);
	}

	@Override
	protected void okPressed() {
		if("".equals(txtSecurityGroupName.getText()) || "".equals(txtDescription.getText())){
			MessageDialog.openError(null,
					Messages.getString("failed"),
					bundle_messages.getString("dialogs.must_fully_input"));
			return;
		}

		securityGroupName = txtSecurityGroupName.getText();
		description = txtDescription.getText();
		
		setReturnCode(OK);
		close();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(bundle_dialog_name.getString("create_security_group"));
	}

	public String getSecurityGroupName() {
		return securityGroupName;
	}

	public String getDescription() {
		return description;
	}
}
