/*
Copyright (C) 2013 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */
package com.clustercontrol.cloud.ui.dialogs;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import com.clustercontrol.cloud.aws.presenter.BlockDeviceMapping;
import com.clustercontrol.cloud.aws.presenter.IInstance;
import com.clustercontrol.cloud.aws.presenter.IStorage;
import com.clustercontrol.cloud.commons.util.MessageManager;
import com.clustercontrol.util.Messages;

public class DetachInstanceDialog extends Dialog {
	private MessageManager bundle_dialog_name = MessageManager.getInstance("dialog_name");
	private MessageManager bundle_dialog_label = MessageManager.getInstance("dialog_label");
	private MessageManager bundle_messages =MessageManager.getInstance("messages");
	private IInstance instance;
	private java.util.List<IStorage> volumes = new ArrayList<IStorage>();
	private Table table;
	private TableViewer tableViewer;
	private Text availabilityZone;
	private Text instanceName;
	
	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public DetachInstanceDialog(Shell parentShell, IInstance instance) {
		super(parentShell);
		this.instance = instance;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);
		container.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		GridLayout gl_container = new GridLayout(2, false);
		container.setLayout(gl_container);
		GridData gd_container = new GridData(GridData.FILL_BOTH);
		gd_container.widthHint = 614;
		container.setLayoutData(gd_container);
		
		Label lblAvailabilityZone = new Label(container, SWT.NONE);
		lblAvailabilityZone.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAvailabilityZone.setText(bundle_dialog_label.getString("dialog.detach_instance.availability_zone_title"));
		
		availabilityZone = new Text(container, SWT.BORDER | SWT.READ_ONLY);
		availabilityZone.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		availabilityZone.setText(instance.getAvailabilityZone());
		
		Label lblInstance = new Label(container, SWT.NONE);
		lblInstance.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblInstance.setText(bundle_dialog_label.getString("dialog.detach_instance.instance_title"));
		
		instanceName = new Text(container, SWT.BORDER | SWT.READ_ONLY);
		instanceName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		instanceName.setText(instance.getInstanceId());
				
		table = new Table(container, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);
		table.setLinesVisible(true);
		table.setHeaderVisible(true);
		table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		TableColumn tblclmnVolumeId = new TableColumn(table, SWT.NONE);
		tblclmnVolumeId.setWidth(90);
		tblclmnVolumeId.setText(bundle_dialog_label.getString("dialog.detach_instance.column.volume_id"));

		TableColumn tableColumn_6 = new TableColumn(table, SWT.NONE);
		tableColumn_6.setWidth(70);
		tableColumn_6.setText(bundle_dialog_label.getString("dialog.detach_instance.column.volume_name"));
		
		TableColumn tableColumn = new TableColumn(table, SWT.NONE);
		tableColumn.setWidth(70);
		tableColumn.setText(bundle_dialog_label.getString("dialog.detach_instance.column.type"));
		
		TableColumn tableColumn_1 = new TableColumn(table, SWT.NONE);
		tableColumn_1.setWidth(70);
		tableColumn_1.setText(bundle_dialog_label.getString("dialog.detach_instance.column.device"));
		
		TableColumn tableColumn_2 = new TableColumn(table, SWT.NONE);
		tableColumn_2.setWidth(70);
		tableColumn_2.setText(bundle_dialog_label.getString("dialog.detach_instance.column.snapshot_id"));
		
		TableColumn tableColumn_3 = new TableColumn(table, SWT.NONE);
		tableColumn_3.setWidth(70);
		tableColumn_3.setText(bundle_dialog_label.getString("dialog.detach_instance.column.size"));
		
		TableColumn tableColumn_4 = new TableColumn(table, SWT.NONE);
		tableColumn_4.setWidth(70);
		tableColumn_4.setText(bundle_dialog_label.getString("dialog.detach_instance.column.volume_type"));
		
		TableColumn tableColumn_5 = new TableColumn(table, SWT.NONE);
		tableColumn_5.setWidth(70);
		tableColumn_5.setText(bundle_dialog_label.getString("dialog.detach_instance.column.iops"));
		
		tableViewer = new TableViewer(table);
		VolumeProvider vp = new VolumeProvider();
		tableViewer.setContentProvider(vp.getVolumeContentProvider());

		
		instance.update();
		instance.getRegion().getStorageRoot().update();
		List<IStorage> storages = new ArrayList<IStorage>();
		for (IStorage s: instance.getRegion().getStorageRoot().getStorages()) {
			for (BlockDeviceMapping d: instance.getBlockDeviceMappings()) {
				if (d.getVolumeId().equals(s.getStorageId())) {
					storages.add(s);
					break;
				}
			}
		}
		tableViewer.setInput(storages);
		tableViewer.setLabelProvider(vp.getVolumeLabelProvider());
		table.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetSelected(SelectionEvent e) {
				volumes.clear();
				for(TableItem ti: table.getSelection()){
					volumes.add((IStorage)ti.getData());
				}
			}
		});
		return area;
	}

	/**
	 * Create contents of the button bar.
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				false);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(640, 480);
	}

	@Override
	protected void okPressed() {
		if(this.volumes.isEmpty()){
			MessageDialog.openError(null,
					Messages.getString("failed"),
					bundle_messages.getString("dialogs.must_fully_input"));
			return;
		}

		Pattern p = Pattern.compile(AvailableDevice.ROOT_DEVICE+".*");
		Boolean b = false;
		for(IStorage vol: volumes){
			if(p.matcher(vol.getStorageAttachment().getDevice()).find()){
				b = true;
				break;
			}
		}
		
		if(b){
			MessageDialog.openError(null,
					Messages.getString("failed"),
					bundle_messages.getString("dialogs.can_not_root_volume"));
			return;
		}		

		setReturnCode(OK);
		close();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(bundle_dialog_name.getString("detach_storage"));
	}

	public java.util.List<String> getVolumes(){
		List<String> storageIds = new ArrayList<String>();
		for (IStorage storage: volumes) {
			storageIds.add(storage.getStorageId());
		}
		return storageIds;
	}
}
